unit MainUnit;

{$mode objfpc}{$H+}


// -----------------------------------------------------------------------------
//
//
//                           latest changes: 2021-05-17
// -----------------------------------------------------------------------------
// some links:
// https://forum.lazarus.freepascal.org/index.php/topic,45486.0.html
//
// -----------------------------------------------------------------------------
// https://forum.lazarus.freepascal.org/index.php/topic,39780.0.html
// "Parenting a form to a TWinControl is a curious, non-standard behaviour."
// "far better to use TFrames or TCustomControls which are designed to need a parent."
// => I don't care this far .. because
//    TPanel = class(TCustomPanel), and
//    TCustomPanel = class(TCustomControl)
// -----------------------------------------------------------------------------


interface

uses
  Interfaces,
  Windows,
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, Menus,
  ExtCtrls, StdCtrls,
  SubUnit, SubUnitGen2;

type

  { TMainForm }

  TMainForm = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Label1: TLabel;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MDIsimulatorPanel: TPanel;
    MenuItemNewChild: TMenuItem;
    SubMenuItem3: TMenuItem;
    StatusBar1: TStatusBar;
    TopPanel: TPanel;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure MDIsimulatorPanelResize(Sender: TObject);
    procedure SubMenuItemNewChildClick(Sender: TObject);
    procedure SubMenuItem3Click(Sender: TObject);
  private
    //
  protected
    procedure NewChildWindow;
  public
    function MDIChildCount: Integer; override;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}


{ TMainForm }

procedure TMainForm.MDIsimulatorPanelResize(Sender: TObject);
var
  L : integer;
begin
  DisableAlign;
  for L:= 0 to Self.MDIsimulatorPanel.ControlCount -1 do
    begin
      //if (MDIsimulatorPanel.Controls[L] is TForm) then
      if (MDIsimulatorPanel.Controls[L] is TChildForm) then
        //if TForm( MDIsimulatorPanel.Controls[L]).WindowState = wsMaximized then
        if TChildForm( MDIsimulatorPanel.Controls[L]).IsMaximized then
          begin
            TForm( MDIsimulatorPanel.Controls[L]).SetBounds( 0, 0,
                                                             MDIsimulatorPanel.Width,// -16,
                                                             MDIsimulatorPanel.Height);// -40);
          end;
    end;
  EnableAlign;
end;


function TMainForm.MDIChildCount: Integer;
var
  L : integer;
begin
  Result:= 0;

  for L:= 0 to Self.MDIsimulatorPanel.ControlCount -1 do
    begin
      if (MDIsimulatorPanel.Controls[L] is TForm) then
        begin
          Result:= Result +1;
        end;
    end;
  //ShowMessage( 'MDIChildCount = ' +IntToStr( Result));
end;


procedure TMainForm.NewChildWindow;
var
  aChildForm: TChildForm;
begin
  aChildForm:= TChildForm.Create( Self);
  Windows.SetParent( aChildForm.handle, Self.Handle);
  aChildForm.Parent:= MDIsimulatorPanel;  // 0.02  added this
  aChildForm.Visible:= TRUE;
  aChildForm.SetBounds( 20 *(MDIChildCount -1),
                        20 *(MDIChildCount -1),
                        min( 350, Self.MDIsimulatorPanel.Width -16),
                        min( 250, Self.MDIsimulatorPanel.Height -39));
  aChildForm.Caption:= aChildForm.Caption +' ' +IntToStr( MDIChildCount);
end;


// --- MainMenu ----------------------------------------------------------------
procedure TMainForm.SubMenuItemNewChildClick(Sender: TObject);
begin
  Self.NewChildWindow;
end;


procedure TMainForm.SubMenuItem3Click(Sender: TObject);
var
  aChildForm: TChildFormGen2;
begin
  aChildForm:= TChildFormGen2.Create( Self);
  Windows.SetParent( aChildForm.handle, Self.Handle);
  aChildForm.Parent:= MDIsimulatorPanel;  // 0.02  added this
  aChildForm.Visible:= TRUE;
  aChildForm.SetBounds( 0, 0, 350, 250);
  aChildForm.Caption:= aChildForm.Caption +' with Windows.SetParent';
end;


// --- Buttons -----------------------------------------------------------------
procedure TMainForm.Button1Click(Sender: TObject);
begin
  Self.SetBounds( 0,
                  0,
                  Screen.Width div 2,
                  Screen.Height div 2);
end;

procedure TMainForm.Button2Click(Sender: TObject);
begin
  Self.SetBounds( Screen.Width div 2 -1,
                  0,
                  Screen.Width div 2 -1,
                  Screen.Height div 2 -1);
end;

procedure TMainForm.Button3Click(Sender: TObject);
begin
  Self.SetBounds( Screen.Width div 2 -1,
                  0,
                  Screen.Width div 2 -1,
                  GetSystemMetrics( SM_CYFULLSCREEN) -1);
end;


end.

